import json
from collections import OrderedDict

import pytest
import requests_mock
from django.http import Http404

from django_healthchecks import views


def check_int():
    return 2


def check_float():
    return 1.5


def test_index_view(rf, settings):
    settings.HEALTH_CHECKS = {
        'database': 'django_healthchecks.contrib.check_dummy_true',
        'redis': 'django_healthchecks.contrib.check_dummy_false',
    }

    request = rf.get('/')
    view = views.HealthCheckView()
    result = view.dispatch(request)

    data = json.loads(result.content.decode(result.charset))
    assert data == {
        'database': True,
        'redis': False,
    }


def test_service_view_bool(rf, settings):
    settings.HEALTH_CHECKS = OrderedDict([
        ('redis', 'django_healthchecks.contrib.check_dummy_false'),
        ('database', 'django_healthchecks.contrib.check_dummy_true'),
    ])

    request = rf.get('/')
    view = views.HealthCheckServiceView()
    result = view.dispatch(request, service='database')

    assert result.status_code == 200
    assert result.content == b'true'


def test_service_view_bytes(rf, settings):
    # This tests the serilization contraints
    settings.HEALTH_CHECKS = OrderedDict([
        ('ip', 'django_healthchecks.contrib.check_remote_addr'),
    ])

    request = rf.get('/')
    view = views.HealthCheckServiceView()
    result = view.dispatch(request, service='ip')

    assert result.status_code == 200
    assert result.content == b'127.0.0.1'


def test_service_view_int(rf, settings):
    # This tests the serilization contraints
    settings.HEALTH_CHECKS = OrderedDict([
        ('val', check_int),
    ])

    request = rf.get('/')
    view = views.HealthCheckServiceView()
    result = view.dispatch(request, service='val')

    assert result.status_code == 200
    assert result['Content-Type'] == 'application/json'
    assert result.content == b'2'


def test_service_view_float(rf, settings):
    # This tests the serilization contraints
    settings.HEALTH_CHECKS = OrderedDict([
        ('val', check_float),
    ])

    request = rf.get('/')
    view = views.HealthCheckServiceView()
    result = view.dispatch(request, service='val')

    assert result.status_code == 200
    assert result['Content-Type'] == 'application/json'
    assert result.content == b'1.5'


def test_service_view_remote(rf, settings):
    settings.HEALTH_CHECKS = {
        'remote_service': 'https://test.com/api/healthchecks/',
    }

    with requests_mock.Mocker() as mock:
        mock.get(
            'https://test.com/api/healthchecks/',
            json={"cache_default": True})

        request = rf.get('/')
        view = views.HealthCheckServiceView()
        result = view.dispatch(request, service='remote_service')

    expected = {'cache_default': True}
    data = json.loads(result.content.decode(result.charset))

    assert result.status_code == 200
    assert result['Content-Type'] == 'application/json'
    assert data == expected


def test_service_view_err(rf, settings):
    settings.HEALTH_CHECKS = {
        'database': 'django_healthchecks.contrib.check_dummy_false'
    }

    request = rf.get('/')
    view = views.HealthCheckServiceView()

    result = view.dispatch(request, service='database')
    assert result.status_code == 200
    assert result.content == b'false'


def test_service_view_err_custom_code(rf, settings):
    settings.HEALTH_CHECKS_ERROR_CODE = 500
    settings.HEALTH_CHECKS = {
        'database': 'django_healthchecks.contrib.check_dummy_false'
    }

    request = rf.get('/')
    view = views.HealthCheckServiceView()

    result = view.dispatch(request, service='database')
    assert result.status_code == 500
    assert result.content == b'false'


def test_service_view_404(rf):
    request = rf.get('/')
    view = views.HealthCheckServiceView()

    with pytest.raises(Http404):
        view.dispatch(request, service='database')


def test_service_require_auth(rf, settings):
    settings.HEALTH_CHECKS = {
        'database': 'django_healthchecks.contrib.check_dummy_true'
    }
    settings.HEALTH_CHECKS_BASIC_AUTH = {
        '*': [('user', 'password')],
    }

    request = rf.get('/')
    view = views.HealthCheckServiceView()

    result = view.dispatch(request, service='database')
    assert result.status_code == 401



from django.conf import settings


def pytest_configure():
    settings.configure(
        HEALTH_CHECKS={},
        MIDDLEWARE_CLASSES=[],
        CACHES={
            'default': {
                'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
                'LOCATION': 'unique-snowflake',
            }
        },
        DATABASES={
            'default': {
                'ENGINE': 'django.db.backends.sqlite3',
                'NAME': 'db.sqlite',
            },
        }
    )



import pytest

from django_healthchecks import contrib
from django.core import cache


@pytest.mark.django_db
def test_check_database():
    assert contrib.check_database()


def test_check_cache_default():
    cache.caches = cache.CacheHandler()
    assert contrib.check_cache_default()


def test_check_cache_default_down(settings):
    """Check if the application can connect to the default cached and
    read/write some dummy data.

    """
    cache.caches = cache.CacheHandler()
    settings.CACHES={
        'default': {
            'BACKEND': 'django.core.cache.backends.dummy.DummyCache',
        }
    }
    assert not contrib.check_cache_default()



import base64
import functools
import inspect
from importlib import import_module

import requests
from django.conf import settings
from django.utils.encoding import force_text


try:
    from django.utils.module_loading import import_string
except ImportError:
    def import_string(value):
        module_name, func_name = value.rsplit('.', 1)
        module = import_module(module_name)
        return getattr(module, func_name)


class PermissionDenied(Exception):
    pass


def create_report(request=None):
    """Run all checks and return a tuple containing results and boolean to
    indicate to indicate if all things are healthy.

    """
    report = {}
    has_error = False

    for service, check_func in _get_check_functions(request=request):
        report[service] = check_func() or False

        if not report[service]:
            has_error = True
    return report, not has_error


def create_service_result(service, request=None):
    functions = list(_get_check_functions(name=service, request=request))
    if not functions:
        return

    check_func = functions[0][1]
    result = check_func() or False
    return result


def _get_check_functions(name=None, request=None):
    checks = _get_registered_health_checks()
    if not checks or (name and name not in checks):
        raise StopIteration()

    checks = _filter_checks_on_permission(request, checks)
    if not checks or (name and name not in checks):
        raise PermissionDenied()

    for service, func_string in checks.items():
        if name and name != service:
            continue

        if callable(func_string):
            check_func = func_string
        elif func_string.startswith(('https://', 'http://')):
            check_func = _http_healthcheck_func(func_string)
        else:
            check_func = import_string(func_string)

        spec = inspect.getargspec(check_func)
        if spec.args == ['request']:
            check_func = functools.partial(check_func, request)

        yield service, check_func


def _get_registered_health_checks():
    return getattr(settings, 'HEALTH_CHECKS', {})


def _http_healthcheck_func(url):
    def handle_remote_request():
        try:
            response = requests.get(url, timeout=_get_http_healthcheck_timeout())
        except requests.exceptions.RequestException:
            return False

        if response.ok:
            return response.json()
        return False

    return handle_remote_request


def _get_http_healthcheck_timeout():
    return getattr(settings, 'HEALTH_CHECKS_HTTP_TIMEOUT', 0.5)


def _filter_checks_on_permission(request, checks):
    permissions = getattr(settings, 'HEALTH_CHECKS_BASIC_AUTH', {})
    if not permissions:
        return checks

    allowed = {}
    for name in checks.keys():
        required_credentials = permissions.get(name, permissions.get('*'))

        if required_credentials:
            credentials = _get_basic_auth(request)
            if not credentials or credentials not in required_credentials:
                continue

        allowed[name] = checks[name]
    return allowed


def _get_basic_auth(request):
    auth = request.META.get('HTTP_AUTHORIZATION')
    if not auth:
        return

    auth = auth.split()
    if len(auth) == 2 and force_text(auth[0]).lower() == u'basic':
        credentials = base64.b64decode(auth[1]).decode('latin-1')
        return tuple(credentials.split(':'))



import uuid

from django.core.cache import cache
from django.db import connection


def check_database():
    """Check if the application can perform a dummy sql query"""
    with connection.cursor() as cursor:
        cursor.execute('SELECT 1; -- Healthcheck')
        row = cursor.fetchone()
    return row[0] == 1


def check_cache_default():
    """Check if the application can connect to the default cached and
    read/write some dummy data.

    """
    dummy = str(uuid.uuid4())
    key = 'healthcheck:%s' % dummy
    cache.set(key, dummy, timeout=5)
    cached_value = cache.get(key)
    return cached_value == dummy


def check_dummy_true():
    return True


def check_dummy_false():
    return False


def check_remote_addr(request):
    return request.META['REMOTE_ADDR']



import base64

from django_healthchecks import checker

import requests
import requests_mock


def test_create_report(settings):
    settings.HEALTH_CHECKS = {
        'database': 'django_healthchecks.contrib.check_dummy_true',
        'remote_service': 'https://test.com/api/healthchecks/',
    }

    with requests_mock.Mocker() as mock:
        mock.get(
            'https://test.com/api/healthchecks/',
            text='{"cache_default": true}')
        result, is_healthy = checker.create_report()

    expected = {
        'database': True,
        'remote_service': {'cache_default': True},
    }

    assert result == expected
    assert is_healthy is True


def test_service_timeout(settings):
    settings.HEALTH_CHECKS = {
        'database': 'django_healthchecks.contrib.check_dummy_true',
        'timeout_service': 'http://timeout.com/api/healthchecks/',
    }

    with requests_mock.Mocker() as mock:
        mock.register_uri(
            'GET', 'http://timeout.com/api/healthchecks/',
            exc=requests.exceptions.Timeout)
        result, is_healthy = checker.create_report()

    expected = {
        'database': True,
        'timeout_service': False
    }

    assert result == expected
    assert is_healthy is False


def test_create_report_err(settings):
    settings.HEALTH_CHECKS = {
        'database': 'django_healthchecks.contrib.check_dummy_true',
        'i_fail': 'django_healthchecks.contrib.check_dummy_false',
    }
    result, is_healthy = checker.create_report()
    expected = {
        'database': True,
        'i_fail': False,
    }

    assert result == expected
    assert is_healthy is False


def test_create_service_result(settings):
    settings.HEALTH_CHECKS = {
        'database': 'django_healthchecks.contrib.check_dummy_true'
    }
    result = checker.create_service_result('database')
    assert result is True


def test_create_service_result_unknown(settings):
    result = checker.create_service_result('database')
    assert result is None


def test_create_with_request(rf, settings):
    settings.HEALTH_CHECKS = {
        'remote_addr': 'django_healthchecks.contrib.check_remote_addr'
    }
    request = rf.get('/')
    result = checker.create_service_result('remote_addr', request)
    assert result == '127.0.0.1'


def test_filter_checks_on_permission(rf, settings):
    checks = {
        'public': 'django_healthchecks.contrib.check_dummy_true',
        'private': 'django_healthchecks.contrib.check_dummy_true'
    }

    settings.HEALTH_CHECKS_BASIC_AUTH = {
        '*': [('user', 'password')],
        'public': [],
    }

    request = rf.get('/')
    result = checker._filter_checks_on_permission(request, checks)
    assert result == {
        'public': 'django_healthchecks.contrib.check_dummy_true',
    }

    request = rf.get(
        '/', HTTP_AUTHORIZATION=b'Basic ' + base64.b64encode(b'user:password'))

    result = checker._filter_checks_on_permission(request, checks)
    assert result == {
        'public': 'django_healthchecks.contrib.check_dummy_true',
        'private': 'django_healthchecks.contrib.check_dummy_true'
    }



from django.conf.urls import url

from django_healthchecks import views

urlpatterns = [
    url(r'^$', views.HealthCheckView.as_view(), name='index'),
    url(r'^(?P<service>.*)$', views.HealthCheckServiceView.as_view(),
        name='service'),
]



import six

from django.conf import settings
from django.http import HttpResponse, JsonResponse
from django.http.response import Http404
from django.views.decorators.cache import cache_control
from django.views.generic import View

from django_healthchecks.checker import create_report, create_service_result
from django_healthchecks.checker import PermissionDenied


class NoCacheMixin(object):

    @classmethod
    def as_view(cls, **kwargs):
        view = super(NoCacheMixin, cls).as_view(**kwargs)
        return cache_control(
            private=True, no_cache=True, no_store=True, max_age=0)(view)


class HealthCheckView(NoCacheMixin, View):

    def get(self, request, *args, **kwargs):
        try:
            report, is_healthy = create_report(request=request)
        except PermissionDenied:
            response = HttpResponse(status=401)
            response['WWW-Authenticate'] = 'Basic realm="Healthchecks"'
            return response
        status_code = 200 if is_healthy else _get_err_status_code()
        return JsonResponse(report, status=status_code)


class HealthCheckServiceView(NoCacheMixin, View):

    def get(self, request, service, *args, **kwargs):
        service_path = list(filter(lambda s: s, service.split('/')))
        service = service_path.pop(0)

        try:
            result = create_service_result(service=service, request=request)
        except PermissionDenied:
            response = HttpResponse(status=401)
            response['WWW-Authenticate'] = 'Basic realm="Healthchecks"'
            return response

        return self.create_result_response(service, result, service_path)

    def create_result_response(self, service, result, service_path):
        for nested in service_path:
            result = result.get(nested, None)
            if result is None:
                break

        if result is None:
            raise Http404()

        if result in (True, False):
            status_code = 200 if result else _get_err_status_code()
            return HttpResponse(str(result).lower(), status=status_code)
        elif isinstance(result, six.string_types) or isinstance(result, bytes):
            return HttpResponse(result)
        else:
            # Django requires safe=False for non-dict values.
            return JsonResponse(result, safe=False)


def _get_err_status_code():
    return getattr(settings, 'HEALTH_CHECKS_ERROR_CODE', 200)
